 /* qtod - convert Fortran program from quad to double precision */
%{
#include "flex.h"
#include "fortran.h"
int debug = 0;				/* non-zero for debug output when */
					/* compiled with -DDEBUG */
int hp_output = 0;			/* non-0 for quad-precision output */
%}
		/* increase lex table sizes for lex */
%a 4000
%e 2000
%k 1500
%n  500
%o 6000
%p 4000

a	[aA]
b	[bB]
c	[cC]
d	[dD]
e	[eE]
f	[fF]
g	[gG]
h	[hH]
i	[iI]
j	[jJ]
k	[kK]
l	[lL]
m	[mM]
n	[nN]
o	[oO]
p	[pP]
q	[qQ]
r	[rR]
s	[sS]
t	[tT]
u	[uU]
v	[vV]
w	[wW]
x	[xX]
y	[yY]
z	[zZ]
				    /* digits with optional embedded blanks */
B		([0-9]|[0-9][0-9 ]*[0-9])
					/* decimal digit */
D		[0-9]
					/* floating-point exponent */
E		[QDEqde][-+]?{B}
					/* optional whitespace */
W		[ \t]*
					/* number */
N		{B}
%%
^[*Cc].*$			ECHO;	/* copy comments unchanged */

^\t[1-9]			{	/* handle tabbed continuation */
				    outstr("     ");
				    output(yytext[1]);
				}

^\t0 |
^\t				outstr("      "); /* handle tabbed initial */

^{W}{r}{e}{a}{l}"*"16		convert_declaration(yytext);

^{W}{i}{m}{p}{l}{i}{c}{i}{t}{W}{r}{e}{a}{l}"*"16	convert_declaration(yytext);

^"     "[^ 0]			ECHO;	/* copy continuation line prefixes */

{N}{p}{N}*[A-Za-z]{N}"."{N}	ECHO;	/* copy FORMAT items intact */

{N}*[A-Za-z]{N}"."{N}		ECHO;	/* copy FORMAT items intact */

"."{n}{o}{t}"." |
"."{a}{n}{d}"." |
"."{x}{o}{r}"." |
"."{o}{r}"." |
"."{g}{e}"." |
"."{g}{t}"." |
"."{l}{e}"." |
"."{l}{t}"." |
"."{e}{q}"." |
"."{e}{q}{v}"." |
"."{n}{e}"." |
"."{n}{e}{q}"."			ECHO;	/* recognize logical operators */

{N}"."{N}*({W}{E})? |
{N}*"."{N}({W}{E})? |
{N}{W}{E}			convert_number(yytext) ;

{i}{q}{i}{n}{t}/{W}"("		cvtstr("idint"); /* handle name conversions */
{i}{q}{n}{i}{n}{t}/{W}"("	cvtstr("idnint");
{q}{e}{x}{t}/{W}"("		cvtstr("dble");
{q}{i}{n}{t}/{W}"("		cvtstr("dint");
{q}{l}{o}{g}/{W}"("		cvtstr("dlog");
{q}{l}{o}{g}10/{W}"("		cvtstr("dlog10");
{q}{m}{a}{x}0/{W}"("		cvtstr("dmax0");
{q}{m}{a}{x}1/{W}"("		cvtstr("dmax1");
{q}{m}{i}{n}0/{W}"("		cvtstr("dmin0");
{q}{m}{i}{n}1/{W}"("		cvtstr("dmin1");
{q}{m}{o}{d}/{W}"("		cvtstr("dmod");
{q}{n}{i}{n}{t}/{W}"("		cvtstr("dnint");
{q}{a}{b}{s}/{W}"("		cvtstr("dabs");
{q}{a}{c}{o}{s}/{W}"("		cvtstr("dacos");
{q}{a}{s}{i}{n}/{W}"("		cvtstr("dasin");
{q}{a}{t}{a}{n}/{W}"("		cvtstr("datan");
{q}{a}{t}{a}{n}2/{W}"("		cvtstr("datan2");
{q}{c}{o}{s}/{W}"("		cvtstr("dcos");
{q}{c}{o}{s}{h}/{W}"("		cvtstr("dcosh");
{q}{d}{i}{m}/{W}"("		cvtstr("ddim");
{q}{e}{x}{p}/{W}"("		cvtstr("dexp");
{q}{f}{l}{o}{a}{t}/{W}"("	cvtstr("dfloat");
{q}{s}{i}{g}{n}/{W}"("		cvtstr("dsign");
{q}{s}{i}{n}/{W}"("		cvtstr("dsin");
{q}{s}{i}{n}{h}/{W}"("		cvtstr("dsinh");
{q}{s}{q}{r}{t}/{W}"("		cvtstr("dsqrt");
{q}{t}{a}{n}/{W}"("		cvtstr("dtan");
{q}{t}{a}{n}{h}/{W}"("		cvtstr("dtanh");

{q}{r}{a}{n}{d}/{W}"("		cvtstr("drand");	/* common extensions */
{q}1{m}{a}{c}{h}/{W}"("		cvtstr("d1mach");

{B}{h}				outhollerith(yytext);

'[^']*(''[^']*)*'		outquotedstring(yytext);

[A-Za-z][A-Za-z0-9]+ |
[0-9]+ |
\n |
.				ECHO;
%%

#include <common.h>
